AdditionalSettingsFrame = {
	RESTART_OPTION_ACTIVE = true,
	CONFIRM_RESTART = true,
	TEST = false,
	CONTROLS = {
		SETTINGS_CONTAINER = "settingsContainer",
		BOX_LAYOUT = "boxLayout",
		HELP_BOX = "additionalSettingsHelpBox",
		HELP_BOX_TEXT = "additionalSettingsHelpBoxText",
		CHECKBOX_HUD = "checkHUD",
		OPTION_CROSSHAIR = "multiCrosshair",
		OPTION_CURRENT_DATE = "multiCurrentDate",
		CHECKBOX_HOUR_FORMAT = "checkHourFormat",
		OPTION_FADE = "multiFade",
		CHECK_DIALOGS = "checkDialogs",
		CHECK_TRAILER_FILL_LEVEL = "checkTrailerFillLevel",
		OPTION_SMOOTH_VEHICLE = "multiSmoothVehicle",
		OPTION_SMOOTH_PLAYER = "multiSmoothPlayer",
		CHECK_EASY_MOTOR_START = "checkEasyMotorStart",
		CHECK_AUTOSTART = "checkAutoStart",
		CHECK_BASIC_MACHINES = "checkBasicMachines",
		OPTION_LIGHTING = "multiLighting",
		OPTION_BRIGHTNESS = "multiBrightness",
		CHECK_DOF = "checkDof",
		CHECK_BIRDS = "checkBirds",
		CHECK_ANIMALS_COLLISIONS = "checkAnimalsCollisions",
		CHECK_CAMERA_COLLISIONS = "checkCameraCollisions",
		CHECK_PAUSE_GAME = "pauseGame"
	}
}

local AdditionalSettingsFrame_mt = Class(AdditionalSettingsFrame, TabbedMenuFrameElement)

function AdditionalSettingsFrame:new(subclass_mt, l10n)
	local subclass_mt = subclass_mt or AdditionalSettingsFrame_mt
	local self = TabbedMenuFrameElement:new(nil, subclass_mt or AdditionalSettingsFrame_mt)

	self:registerControls(AdditionalSettingsFrame.CONTROLS)

	self.l10n = l10n
	self.isDirty = false
	self.hasCustomMenuButtons = true

	self.checkboxMapping = {}
	self.optionMapping = {}

	return self
end

function AdditionalSettingsFrame:getMainElementSize()
	return self.settingsContainer.size
end

function AdditionalSettingsFrame:getMainElementPosition()
	return self.settingsContainer.absPosition
end

function AdditionalSettingsFrame:onFrameOpen(element)
	AdditionalSettingsFrame:superClass().onFrameOpen(self)

	self:setDisabledState()
	self:updateAdditionalSettings()
	self.isDirty = false

	self.boxLayout:invalidateLayout()
	FocusManager:setFocus(self.boxLayout)
end

function AdditionalSettingsFrame:onFrameClose()
	AdditionalSettingsFrame:superClass().onFrameClose(self)

	self:saveSettingsToXMLFile()
end

function AdditionalSettingsFrame:onGuiSetupFinished()
	AdditionalSettingsFrame:superClass().onGuiSetupFinished(self)
end

function AdditionalSettingsFrame:copyAttributes(src)
	AdditionalSettingsFrame:superClass().copyAttributes(self, src)

	self.l10n = src.l10n
end

function AdditionalSettingsFrame:initialize(settings)
	local addittionalSettings = settings.addittionalSettings
	local currentMission = settings.currentMission

	self.multiCrosshair:setTexts(settings:getCrosshairTexts())
	self.multiCurrentDate:setTexts(settings:getCurrentDateTexts())
	self.multiFade:setTexts(settings:getFadeTexts())
	self.multiSmoothVehicle:setTexts(settings:getSmoothCameraTexts())
	self.multiSmoothPlayer:setTexts(settings:getSmoothCameraTexts())
	self.multiLighting:setTexts(settings:getLightingTexts())
	self.multiLighting.toolTipText = string.format(self.multiLighting.toolTipText, settings.lightingDirectory)
	self.multiBrightness:setTexts(settings:getBrightnessTexts())

	self.checkAnimalsCollisions:setVisible(not currentMission.missionDynamicInfo.isMultiplayer)
	self.checkCameraCollisions:setDisabled(g_modIsLoaded["FS19_disableVehicleCameraCollision"])
	self.pauseGame:setVisible(not currentMission.missionDynamicInfo.isMultiplayer and AdditionalSettingsFrame.TEST)

	self.checkboxMapping[self.checkHUD] = addittionalSettings.hud
	self.checkboxMapping[self.checkHourFormat] = addittionalSettings.hourFormat
	self.checkboxMapping[self.checkDialogs] = addittionalSettings.dialogs
	self.checkboxMapping[self.checkTrailerFillLevel] = addittionalSettings.trailerFillLevel
	self.checkboxMapping[self.checkEasyMotorStart] = addittionalSettings.easyMotorStart
	self.checkboxMapping[self.checkAutoStart] = addittionalSettings.autoStart
	self.checkboxMapping[self.checkBasicMachines] = addittionalSettings.basicMachines
	self.checkboxMapping[self.checkDof] = addittionalSettings.dof
	self.checkboxMapping[self.checkBirds] = addittionalSettings.birds
	self.checkboxMapping[self.checkAnimalsCollisions] = addittionalSettings.animalsCollisions
	self.checkboxMapping[self.checkCameraCollisions] = addittionalSettings.cameraCollisions
	self.checkboxMapping[self.pauseGame] = addittionalSettings.pauseGame

	self.optionMapping[self.multiCrosshair] = addittionalSettings.crosshair
	self.optionMapping[self.multiCurrentDate] = addittionalSettings.currentDate
	self.optionMapping[self.multiFade] = addittionalSettings.fadeEffect
	self.optionMapping[self.multiSmoothVehicle] = addittionalSettings.smoothVehicleCamera
	self.optionMapping[self.multiSmoothPlayer] = addittionalSettings.smoothPlayerCamera
	self.optionMapping[self.multiLighting] = addittionalSettings.lighting
	self.optionMapping[self.multiBrightness] = addittionalSettings.brightness

	self.settings = settings
	self.currentMission = currentMission

	self.backButtonInfo = {
		inputAction = InputAction.MENU_BACK
	}
	self.saveButton = {
		showWhenPaused = true,
		inputAction = InputAction.MENU_ACTIVATE
	}
	self.quitButton = {
		showWhenPaused = true,
		inputAction = InputAction.MENU_CANCEL,
		text = self.l10n:getText(InGameMenu.L10N_SYMBOL.BUTTON_CANCEL_GAME)
	}
	self.restartGameButton = {
		inputAction = InputAction.MENU_EXTRA_1,
		text = self.l10n:getText(AdditionalSettingsFrame.L10N_SYMBOL.BUTTON_RESTART_GAME),
		callback = function ()
			self:onButtonRestartGame()
		end
	}

	self:updateButtons()
end

function AdditionalSettingsFrame:saveSettingsToXMLFile()
	if self.isDirty then
		self.settings:saveSettingsToXMLFile(g_modsSettingsPath)
		self.isDirty = false
	end
end

function AdditionalSettingsFrame:onButtonRestartGame()
	if AdditionalSettingsFrame.CONFIRM_RESTART then
		local dialog = self.settings.gui:showDialog("RestartGameDialog")

		if dialog ~= nil then
			dialog.target:setCallback(self.onYesNoRestartGame, self)
			dialog.target:setText(self.l10n:getText(AdditionalSettingsFrame.L10N_SYMBOL.DIALOG_RESTART_GAME))
		end
	else
		self:restartGame()
	end
end

function AdditionalSettingsFrame:onYesNoRestartGame(state)
	if state == RestartGameDialog.SAVE then
		self.requestGameRestart = true
		self.currentMission.inGameMenu:onButtonSaveGame()
	elseif state == RestartGameDialog.NO_SAVE then
		self:restartGame()
	end
end

function AdditionalSettingsFrame.notifySaveComplete(object)
	local pageAdditionalSettings = object.pageAdditionalSettings

	pageAdditionalSettings:saveSettingsToXMLFile()

	if pageAdditionalSettings.requestGameRestart then
		pageAdditionalSettings.requestGameRestart = nil
		pageAdditionalSettings:restartGame()
	else
		pageAdditionalSettings:updateButtons()
	end
end

function AdditionalSettingsFrame:restartGame()
	restartApplication(" -autoStartSavegameId " .. self.currentMission.missionInfo.savegameIndex)
end

function AdditionalSettingsFrame:updateButtons()
	self.menuButtonInfo = {
		self.backButtonInfo,
		self.saveButton,
		self.quitButton
	}

	if AdditionalSettingsFrame.RESTART_OPTION_ACTIVE and self.currentMission.missionInfo.isValid and not self.currentMission.missionDynamicInfo.isMultiplayer then
		table.insert(self.menuButtonInfo, self.restartGameButton)
	end

	self:setMenuButtonInfoDirty()
end

function AdditionalSettingsFrame:setDisabledState()
	self.multiCrosshair:setDisabled(not self.currentMission.hud:getIsVisible() or g_flightAndNoHUDKeysEnabled)
	self.multiCurrentDate:setDisabled(not self.currentMission.hud:getIsVisible())
	self.checkEasyMotorStart:setDisabled(self.currentMission.missionInfo.automaticMotorStartEnabled)
end

function AdditionalSettingsFrame:updateAdditionalSettings()
	for element, settingsKey in pairs(self.checkboxMapping) do
		self.settings.callFunction("update", settingsKey, element)
		element:setIsChecked(settingsKey.active)
	end

	for element, settingsKey in pairs(self.optionMapping) do
		self.settings.callFunction("update", settingsKey, element)
		element:setState(settingsKey.state + 1)
	end
end

function AdditionalSettingsFrame:updateToolTipBoxVisibility()
	local hasText = self.additionalSettingsHelpBoxText.text ~= nil and self.additionalSettingsHelpBoxText.text ~= ""

	self.additionalSettingsHelpBox:setVisible(hasText)
end

function AdditionalSettingsFrame:onToolTipBoxTextChanged(element, text)
	self:updateToolTipBoxVisibility()
end

function AdditionalSettingsFrame:onClickCheckbox(state, checkboxElement)
	local checkboxMapping = self.checkboxMapping[checkboxElement]

	if checkboxMapping ~= nil then
		local newState = state == CheckedOptionElement.STATE_CHECKED

		checkboxMapping.active = newState
		self.settings.callFunction("callback", checkboxMapping, newState, checkboxElement)

		self:setDisabledState()
		self.isDirty = true
	else
		print("Warning: Invalid settings checkbox event or key configuration for element " .. checkboxElement:toString())
	end
end

function AdditionalSettingsFrame:onClickMultiOption(state, optionElement)
	local optionMapping = self.optionMapping[optionElement]

	if optionMapping ~= nil then
		local newState = state - 1

		optionMapping.state = newState
		self.settings.callFunction("callback", optionMapping, newState, optionElement)

		self:setDisabledState()
		self.isDirty = true
	else
		print("Warning: Invalid settings multi option event or key configuration for element " .. optionElement:toString())
	end
end

AdditionalSettingsFrame.L10N_SYMBOL = {
	BUTTON_RESTART_GAME = "additionalSettings_button_restartGame",
	DIALOG_RESTART_GAME = "additionalSettings_dialog_restartGame"
}